#include "Math.h"
#include "Kalman.h"
#include "Step_CYC10_I2C.h"
#include "Step_CYC10_PWM.h"
#define   MPU6050_I2C_ADDR                0x68
#include "HC_SR04.h"
#include "encoder.h"
#include "OLED_Driver.h"	 
#include "M10LCD.h"
#include "ADC081S101.h"
constexpr uint8_t MPU6050_POWER_CTL     = 0x6B;//电源管理寄存器的地址
constexpr uint8_t MPU6050_XYZ_GYRO      = 0x1B;//陀螺仪配置寄存器的地址
constexpr uint8_t MPU6050_XYZ_ACCEL     = 0x1C;//加速度传感器配置寄存器的地址
constexpr uint8_t MPU6050_FIFO_CTL      = 0x23;//FIFO使能寄存器的地址
constexpr uint8_t MPU6050_XYZ_GYRO_F    = 0x19;//陀螺仪采样率分频寄存器的地址
constexpr uint8_t MPU6050_INT_ENABLE    = 0x38;//中断使能寄存器的地址（引脚INT）
constexpr uint8_t MPU6050_USER_CTRL     = 0x6A;//用户控制寄存器的地址（引脚XDA,XCL）
//constexpr uint8_t ADXL345_INT_ENABLE  = 0x2E;//寄存器的地址
//constexpr uint8_t ADXL345_INT_MAP     = 0x2F;//寄存器的地址

//声明变量
Kalman kalmanRoll; //Roll角滤波器
Kalman kalmanPitch; //Pitch角滤波器
Kalman kalmanYaw; //
float fRad2Deg  = 57.295779513f; //将弧度转为角度的乘数
float caldata[6]               ;
float Balance_Pwm              ;
float moto1                    ;
float moto2                    ;
unsigned long lasttime;
float lastroll  = 0.0f         ; //上一次滤波得到的Roll角
float lastpitch = 0.0f         ; //上一次滤波得到的Pitch角
float lastyaw   = 0.0f         ; //上一次滤波得到的yaw角
float Encoder_Left ;
float Encoder_Right;
float Velocity_Pwm ;
float Turn_Pwm_1,Turn_Pwm_2 ;

void MPU6050_init()
{  
    I2C.masterWrite(MPU6050_I2C_ADDR, MPU6050_POWER_CTL, 0x40);//复位电源管理寄存器
    I2C.masterWrite(MPU6050_I2C_ADDR, MPU6050_POWER_CTL, 0x00);//唤醒电源管理寄存器
    I2C.masterWrite(MPU6050_I2C_ADDR, MPU6050_XYZ_GYRO , 0x18);//陀螺仪量程
    I2C.masterWrite(MPU6050_I2C_ADDR, MPU6050_XYZ_ACCEL, 0x0 );//加速度计量程

}

void get_MPU6050_XYZ_ACCEL(int16_t *pX, int16_t *pY, int16_t *pZ,int16_t *gY)//获取3轴数据
{
    uint8_t buf1[6];
    
    uint8_t buf3[2];
    
    int16_t temp;
    
     I2C.masterRead(MPU6050_I2C_ADDR, 0x3B, buf1, 6);
        
     temp = ((int16_t)buf1[0] << 8) | buf1[1];
   
        *pX = temp;
   
     temp = ((int16_t)buf1[2] << 8) | buf1[3]; 
   
        *pY = temp;
        
     temp = ((int16_t)buf1[4] << 8) | buf1[5] ;
   
        *pZ = temp;  

     I2C.masterRead(MPU6050_I2C_ADDR, 0x45, buf3, 2);
   
     temp = ((int16_t)buf3[0] << 8) | buf3[1]; 
   
        *gY = temp;
        
 
} 
/*
void Calibration()
{
int16_t x, y, z, gx, gy, gz;
float valSums[6] = {0.0f,0.0f,0.0f,0.0f,0.0f,0.0f};
int16_t val[6][2000];

for (int j = 0; j < 2000; ++j)
{
    get_MPU6050_XYZ_ACCEL(&x, &y, &z, &gy);
    val[0][j] = x;
    val[1][j] = y;
    val[2][j] = z;
    val[3][j] = gx;
    val[4][j] = gy;
    val[5][j] = gz;
    valSums[0]   = valSums[0]+ val[0][j];
    valSums[1]   = valSums[1]+ val[1][j];
    valSums[2]   = valSums[2]+ val[2][j];
    valSums[3]   = valSums[3]+ val[3][j];
    valSums[4]   = valSums[4]+ val[4][j];
    valSums[5]   = valSums[5]+ val[5][j];
}
for (int i = 0; i < 6; ++i)
 {
    caldata[i] = (valSums[i] / 2000); 
 }
  caldata[2] -= 16384; //设芯片Z轴竖直向下，设定静态工作点。
}
*/
float SR04_V ()
{	
	float flag;
	float dis_cm,dis_time_us;
	HCSR04.encontrol(0x03);
    dis_time_us = HCSR04.distimeus();
	dis_cm = (dis_time_us * 17 )/1000; //把回波时间换算成cm 
	if(dis_cm<55               ) flag = 2;
	if(dis_cm>=55 && dis_cm <85) flag = 1;
	if(dis_cm>=85              ) flag = 0;
	float  SR04_V = flag;
	return SR04_V;	
}



float balance(float Angle,float Gyro)
{  
   float Bias, kp=210,kd = 2;
   float balance;
   Bias=Angle;      
   balance=kp*Bias+Gyro*kd; 
   return balance;
}

float velocity(float Encoder_left,float Encoder_right,uint8_t signal)
{  
    static float Velocity,Encoder_Least,Encoder,Movement;
	static float Encoder_Integral,Target_Velocity,yaokong;
	static uint8_t flag;
	float kp=50,ki=0.25;
 
    if(signal == 'B')
		flag = 0;
	else if(signal == 'N' )//遥控
		flag = 1;
 
	if(signal == 'W' && flag == 1)  //前进
		yaokong = 40;
	else if (signal == 'S' && flag == 1)//后退
		yaokong = -40;
	else if ((signal == 'I' ||signal == 'K' )&&(flag ==1))
		yaokong = 0;
	
	if(flag == 0)
        yaokong = 30;
		
		
	Encoder_Least =(Encoder_Left+Encoder_Right);                
	Encoder *= 0.8;		                                           
	Encoder += Encoder_Least*0.2;	                               
	Encoder_Integral +=Encoder;                                    
	Encoder_Integral  =Encoder_Integral+yaokong;                    
	if(Encoder_Integral>10000)  	Encoder_Integral=10000;        
	if(Encoder_Integral<-10000)	    Encoder_Integral=-10000;           
	Velocity=Encoder*kp+Encoder_Integral*ki;                       
	return Velocity;
}

void Set_Pwm(float moto1,float moto2)
{
      if(moto1 > 0)    
	  GPIO_P0 = 0x09;
	  //Serial.println("fuck son of bitch ");}
      else  if(moto1<0)
	  { GPIO_P0 = 0x06;
      moto1 = - moto1;
	  moto2 = - moto2;}
	 //  Serial.println("fuck bitch "); }
	  else 
		GPIO_P0 = 0x0F;
	  
	  if(moto1 >4500)
	    {moto1= 4500;
	     moto2= 4500;}
      PWM.resolution(0,1);
      PWM.dutyCycle(0,(uint16_t) (moto1),5000-(uint16_t)(moto1));
      PWM.resolution(1,1);
      PWM.dutyCycle(1,(uint16_t)(moto2),5000-(uint16_t)(moto2));
}
   
void setup() {
    Serial.println ("fuck");
    MPU6050_init();
	delay(1000);
    //Calibration();
    lasttime=micros();
	OLED.init();
	OLED.clear();
	Serial2.begin(115200);
/*
	  GPIO_P0 = 0x06;
	  PWM.resolution(0,1);
      PWM.dutyCycle(0,5200,1000);
      PWM.resolution(1,1);
      PWM.dutyCycle(1,5200,1000);
*/
}

void loop() 
{
  unsigned long curtime = micros ();
  int16_t x, y, z, gx, gy, gz;
  uint16_t dis_time_us;
  uint32_t encodercnt1;
  uint32_t encodercnt_A;
  uint32_t encodercnt_B;
  uint16_t encoder_A;
  uint16_t encoder_B;
  float Source_Voltage;
  float flag;
  static  uint8_t signal;
  static float flag1;
  get_MPU6050_XYZ_ACCEL (&x, &y, &z,&gy);
  signal = Serial2.read ();
//换算量程，纠正读数
  x = float(x  - 3214.40)  /1638.4f;// - caldata[0])/1638.4f;
  y = float(y  + 57.86)    /1638.4f;// - caldata[1])/1638.4f;
  z = float(z  + 13446.20 )/1638.4f;// - caldata[2])/1638.4f;

  gy =float(gy +     1)    /16.384f;// - caldata[4])/16.384f;
  gy =  -gy;

  
//算pitch角和roll角 
  float fNorm   = sqrt(x *x + y * y + z * z);
  float fNormYZ = sqrt(y *y + z * z);
  float fCos2   = fNormYZ / fNorm;
  float pitch   = acos(fCos2) * fRad2Deg;
  if( x<0 )
  {pitch = -pitch;}


//滤波

  float dt              = (double)(curtime - lasttime)/1000000.0;

  float newpitch        = kalmanPitch.getAngle(pitch, gy, dt);

  float pitchrate       = (newpitch - lastpitch)/dt;

//更新
  
  lastpitch  = newpitch;


//判断转速方向
  encodercnt1 = encoder1.encodercnt();
  encodercnt_A = encodercnt1;
  encodercnt_B = encodercnt1;
  encoder_A =  encodercnt_A & 0x0000ffff;
  encoder_B = (encodercnt_B & 0xffff0000) >> 16;

  if(moto1>=0){
	Encoder_Left   =     -encoder_A;
	Encoder_Right  =     -encoder_B;
  }
  else{
	Encoder_Left   =      encoder_A;
	Encoder_Right  =      encoder_B;
  }

  
    flag = SR04_V ();
	if(flag==0) Turn_Pwm_1 = 0;
	  else if(flag==1) Turn_Pwm_1 = 100;
	    else if(flag==2) Turn_Pwm_1 = 200;

	
	
	if (signal == 'B' )
	flag1 = 0;
	else if (signal == 'N' )
	flag1  = 1; 
	
    if (signal == 'A' && flag1 ==1 )
	Turn_Pwm_2 = 150;
    else if (signal == 'D'&&flag1 == 1)
	Turn_Pwm_2 = -150;
    else if((signal == 'J'||signal == 'L')&&(flag1 == 1))
    Turn_Pwm_2= 0;
 
    if(flag1 == 0)
	Turn_Pwm_2 = Turn_Pwm_1;	

    Balance_Pwm =balance(newpitch,pitchrate);   
    Velocity_Pwm=velocity(Encoder_Left,Encoder_Right,signal); 
	

   
    moto1=Balance_Pwm-Velocity_Pwm+Turn_Pwm_2;
    moto2=Balance_Pwm-Velocity_Pwm-Turn_Pwm_2;
	
//检测是否倒下
	if(newpitch>30 || newpitch <-30)  //28  -30
	{
		moto1 = 0;
		moto2 = 0;
	};
	
    Set_Pwm(moto1,moto2); 
    Source_Voltage=adc.ADC081S();
	//Serial.println(Source_Voltage);
	Source_Voltage=(float)(Source_Voltage/(17.07f));
	float Source_Voltage1 = Source_Voltage;
	int zhengshu = (int)(Source_Voltage);
	int xiaoshu  = (int)( (Source_Voltage1 - zhengshu )*100);
	OLED.showChar(26,1,'V');
	OLED.showChar(36,1,'o');
	OLED.showChar(46,1,'l');
	OLED.showChar(56,1,'t');
	OLED.showChar(66,1,'a');
	OLED.showChar(76,1,'g');
	OLED.showChar(86,1,'e');
    OLED.showNum(32,4,zhengshu,2,16);
	OLED.showChar (50,4,'.');
	OLED.showNum(58,4,xiaoshu,2,16);
	OLED.showChar(80,4,'V');
	OLED.showChar(70,7,signal);	
	
	Serial.print(newpitch);   
    Serial.print ("(");
    Serial.print (pitchrate);	
	Serial.print ("),\n");
                      
	
/*
    Serial.println(z);
	Serial.println(x);
	Serial.println(y);*/
	//Serial.println(newpitch);
	encodercnt_A = encodercnt_A & 0x0000ffff;
	//Serial.print("encodercnt_A = ");
	//Serial.print(encoder_A);
	encodercnt_B = (encodercnt_B & 0xffff0000) >> 16;
	//Serial.print("encodercnt_B = ");
	//Serial.println(encoder_B);
	
    /*
  Serial.print (roll);
  Serial.print ("\t roll角速度:");
  Serial.println (rollrate);
  Serial.print ("pitch角:");
  Serial.print(newpitch);
  Serial.print ("\t pitch角速度:");
  Serial.println (pitchrate);
  Serial.print ("yaw角:");
  Serial.print (newyaw);
  Serial.print ("\t yaw角速度:");
  Serial.println (yawrate);
  delay(1000);
  
  Serial.print ("pitch角:");
  Serial.println(newpitch);
  Serial.println ("\t pitch角速度:");
  Serial.println (pitchrate);
  delay(100);*/

 //Serial.println (moto1);
	lasttime = curtime;
	delay(5);

}